@@ -138,7 +138,7 @@ class NmtBin(object):
             will be multiplied by `ell * (ell + 1) / 2 * PI`, where `ell` \
             is the multipole order (no prefactor otherwise).
         """
-        nls = np.amax(ell_end)
+        nls = int(np.amax(ell_end))
         ells, bpws, weights = [], [], []
         for ib, (li, le) in enumerate(zip(ell_ini, ell_end)):
             nlb = int(le - li)
@@ -156,7 +156,8 @@ class NmtBin(object):
 
     def __del__(self):
         if self.bin is not None:
-            lib.bins_free(self.bin)
+            if lib.bins_free is not None:
+                lib.bins_free(self.bin)
             self.bin = None
 
     def get_n_bands(self):
@@ -287,7 +288,8 @@ class NmtBinFlat(object):
 
     def __del__(self):
         if self.bin is not None:
-            lib.bins_flat_free(self.bin)
+            if lib.bins_flat_free is not None:
+                lib.bins_flat_free(self.bin)
             self.bin = None
 
     def get_n_bands(self):
