@@ -1083,7 +1083,7 @@ CTEST(nmt,master_errors) {
   set_error_policy(THROW_ON_ERROR);
 
   //Read from non-existent file
-  try { w=nmt_workspace_read_fits("nofile.fits"); }
+  try { w=nmt_workspace_read_fits("nofile.fits",1); }
   ASSERT_NOT_EQUAL(0,nmt_exception_status);
   ASSERT_NULL(w);
   //Wrong bins
